#include <Rcpp.h>
#include <cmath>
#define SMALL 1e-300
#define LARGE 1e300
//kap eta [-50 +20]
//----------------------------Global Variables------------------------------//
//In R, declare size_tseq somewhere, size_tseq<-length(tseq);
//In R there should be global variable, xn, ttalln, malln, Kn, Q, nn, ispmt
// -------------------------Global Variables End-------------------------- //

using namespace Rcpp;

//NumericVector is the vector type in Rcpp, vec is the vector type in RcppArmadillo
//Rcpp does not support matrix manipulation, mat is the matrix type in RcppArmadillo
//Default vec is colume vector (colvec)
//as function can be used for type conversion

//NumericVector is similar to std::vector

//sugar functions are functions that support operation on vectors

inline NumericVector L1c(NumericVector lxi, NumericVector malli, NumericVector indti, NumericMatrix ispmt){
	// ^2 and add 0.0
	int Ki = malli.length();
	int size_tseq = ispmt.ncol();
	NumericVector xi(lxi.length() + 1);
	NumericVector::iterator it_xi = xi.begin();
	*it_xi = 0.0;
	it_xi++;
	for (NumericVector::iterator it_lxi1=lxi.begin(); it_lxi1 != lxi.end(); it_lxi1++, it_xi++)
	{
		double sumexp = 0.0;
		for (NumericVector::iterator it_lxi2 = lxi.begin(); it_lxi2 != lxi.end(); it_lxi2++)
		{
			sumexp += exp(*it_lxi2 - *it_lxi1);
		}
		*it_xi = 1 / sumexp;
	}
	// 	indti = pmin(indti, size_tseq - 1);
	for (NumericVector::iterator it = indti.begin(); it != indti.end(); it++)
	{
		if ((*it) > (size_tseq - 1))
		{
			*it = size_tseq - 1;
		}
	}
	//xiv*ispmti.col(Ki) + SMALL
	//Rcout << Ki << std::endl;
	//Rcout << indti[Ki] << std::endl;
	NumericVector colKi = ispmt.column(indti[Ki]);
	double dvalue = SMALL;
	for (std::size_t i = 0; i < xi.length(); i++)
	{
		dvalue += xi[i] * colKi[i];
	}
	//temp = xiv*(ispmti.cols(1, Ki) - ispmti.cols(0, Ki - 1) + SMALL)
	//pow(temp/dvalue, malli)
	NumericVector temp(Ki, SMALL);//length Ki, filled with 0;
	for (std::size_t i = 0; i < Ki; i++)
	{
		NumericVector col0 = ispmt.column(indti[i]);
		NumericVector col1 = ispmt.column(indti[i + 1]);
		for (std::size_t j = 0; j < col1.length(); j++)
		{
			temp[i] += xi[j] * (col1[j] - col0[j]);
		}
		temp[i] = std::pow(temp[i] / dvalue, malli[i]) + SMALL;
	}
	return(temp);
}

// [[Rcpp::export]]
double negPnc(NumericVector kexi){
	// global varaibles in R 
	Environment env = Environment::global_env();
	NumericMatrix xn = env["xn"];
	List ttalln = env["ttalln"];
	List malln = env["malln"];
	int Q = env["Q"];
	int nn = env["nn"];
	int pn = env["pn"];
	NumericMatrix ispmt = env["ispmt"];
	double kap = kexi[0];
	double expkap = exp(kap);
	NumericVector eta(kexi.begin() + 1, kexi.begin() + pn + 1);
	NumericVector lxi(kexi.begin() + pn + 1, kexi.end());

	if (kap > 20 || kap < -50)
		return LARGE;
	if (eta[0]> 20 || eta[0] < -50)
		return LARGE;
	if (eta[1]> 20 || eta[1] < -50)
		return LARGE;
	if (eta[2]> 20 || eta[2] < -50)
		return LARGE;
	if (eta[3]> 20 || eta[3] < -50)
		return LARGE;
	int ncol = xn.ncol();
	double suml = 0.0;
	for (int i = 0; i < nn; i++)
	{
		double Exi = SMALL;
		double scale = 0.0;
		for (int j = 0; j < ncol; j++)
			scale += eta[j] * xn(i, j);
		scale = exp(scale);
		NumericVector ttallni = ttalln[i];
		for (int q = 0; q < Q; q++)
		{
			double Atempq = R::qgamma(double(q + 1) / (Q + 1), expkap, scale, 1, 0);
			Atempq = round(100.0 * Atempq);
			NumericVector indti = clone<NumericVector>(ttallni);
			for (NumericVector::iterator it = indti.begin(); it != indti.end(); it++)
			{
				*it = round((*it) * 100.0 + Atempq);
			}
			indti.push_front(0.0);
			NumericVector temp = L1c(lxi, malln[i], indti, ispmt); // Kn[i] = malln[i].length()
			Exi += std::accumulate(temp.begin(), temp.end(), 1.0, std::multiplies<double>()); // prod
		}
		suml -= std::log(Exi / (Q+1));
	}
	return(suml);
}
